@@ -2292,8 +2292,12 @@ new_engine_cb (GObject             *obj,
                                              "Opertation was cancelled");
         }
         else {
-            /* Let BusEngineProxy call a Disable signal. */
-            bus_input_context_disable (data->context);
+            /* Let BusEngineProxy call a Disable signal.
+             * The Disable signal is needed when the engines are switched
+             * but not needed when an engine is enabled initially. */
+            if (data->context->engine) {
+                bus_input_context_disable (data->context);
+            }
             bus_input_context_set_engine (data->context, engine);
             g_object_unref (engine);
             bus_input_context_enable (data->context);
